// Mise en oeuvre d'un composant master MM pour l'ADC
// Standard sequencer with avalon-MM external sample storage
// Ce module gère les canaux 1 et 3 
//
module avalon_ADC (
	input				sys_clk,
	input				reset_n,	
	
	//avalon_master pilotage ADC
	output	[15:0]		avalon_address,
	output				avalon_read,
	input				avalon_waitrequest,
	input	[31:0]		avalon_readdata,
	output				avalon_write,
	output	[31:0]		avalon_writedata,
	
	// avalon streaming sink reception données ADC
	input				response_valid,
	input	[4:0]		response_channel,
	input	[11:0]		response_data,
	input				response_startofpacket,
	input				response_endofpacket,

	// bits [11:0] Valeur de l'ADC
	// bits [16:12] canal de l'ADC
	output	[16:0]		ADC_value	
	
);

reg [2:0]  reg_state; 
reg [11:0] data_lues;
reg [4:0]  canal;

// machine à états pour config de l'ADC
`define ADC_IDLE 		3'b000
`define ADC_START	3'b001
`define ADC_RUN 	3'b010

// Adr base de l'ADC
`define ADC_CMD_ADDR 		16'h0000

initial
begin
	reg_state <= `ADC_IDLE;
	data_lues <= 12'h00;
	canal <= 4'b0;
end

always @(posedge sys_clk or negedge reset_n)
begin
	if (!reset_n) 
 	begin
 		reg_state <= `ADC_IDLE;
 	end
	else
	begin
		case (reg_state)
		    `ADC_IDLE:		
		    begin
				reg_state <= `ADC_START;
				avalon_address <= `ADC_CMD_ADDR; // le séquenceur est piloté par Avalon MM
				avalon_read <= 1'b0;
				avalon_write <= 1'b1;
				avalon_writedata <= 32'b0;
		   	end
		    `ADC_START:	
		    begin
				reg_state <= `ADC_RUN;
				avalon_address <= `ADC_CMD_ADDR;
				avalon_read <= 1'b0;
				avalon_write <= 1'b1;
				avalon_writedata <= 32'h0001;
		  		end
		    `ADC_RUN: 
		    begin
				reg_state <= `ADC_RUN;				// boucler sur RUN
				avalon_address <= `ADC_CMD_ADDR;
				avalon_read <= 1'b0;
				avalon_write <= 1'b0;
    		end
		endcase	
	end		
end

always @ (posedge response_valid)
begin
	data_lues[11:0] = response_data[11:0];
	canal[4:0] =  response_channel[4:0];
end

assign ADC_value[11:0] = data_lues[11:0];
assign ADC_value[16:12] = canal[4:0];

endmodule